<?php
namespace HCM\AVL\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repository;

class GerenciamentoRepository {
    
    protected  $connection;
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
    }
    
    public function updateParticipaval($params) {
        $query = $this->entityManager->getConnection()->prepare("
            UPDATE PARTICIPAVAL
                       SET IDSITUACAOAVAL = :P_IDSITUACAOAVAL, DTCONCLUSAOAVAL = :P_DTCONCLUSAOAVAL,
                           VRRESULTAVALIA = :P_VRRESULTAVALIA, CDTIPORESU = :P_CDTIPORESU
                     WHERE NRPARCEXECAVAL = :P_NRPARCEXECAVAL
                       AND NRPARCAVALIADO = :P_NRPARCAVALIADO
                       AND NRCOMBQUESTAVAL = :P_NRCOMBQUESTAVAL
                       AND NRORG = :P_NRORG"
        );

        return $query->execute($params);;
    }
    
    public function deleteRespavaliacao($params) {
        $query = $this->entityManager->getConnection()->prepare("
            DELETE
               FROM RESPAVALIACAO
              WHERE EXISTS (SELECT * FROM EXECAVALIACAO, RESPAVALIACAO T1
              WHERE EXECAVALIACAO.NREXECAVALIACAO = T1.NREXECAVALIACAO
                AND EXECAVALIACAO.NREXECAVALIACAO = RESPAVALIACAO.NREXECAVALIACAO
                AND T1.NREXECAVALIACAO            = RESPAVALIACAO.NREXECAVALIACAO
                AND EXECAVALIACAO.NRAVALIACAO     = :P_NRAVALIACAO
                AND EXECAVALIACAO.NRCOMBQUESTAVAL = :P_NRCOMBQUESTAVAL
                AND EXECAVALIACAO.NRPARCEXECAVAL  = :P_NRPARCEXECAVAL
                AND EXECAVALIACAO.NRPARCAVALIADO  = :P_NRPARCAVALIADO)"
        );

        return $query->execute($params);;
    }
    
    public function deleteExecavaliacao($params) {
        $query = $this->entityManager->getConnection()->prepare("
            DELETE 
                FROM EXECAVALIACAO T1
              WHERE EXISTS (SELECT *
                              FROM  EXECAVALIACAO, AVALIACAOGERAL
                             WHERE EXECAVALIACAO.NRAVALIACAO = AVALIACAOGERAL.NRAVALIACAO
                AND EXECAVALIACAO.NREXECAVALIACAO = T1.NREXECAVALIACAO
                AND EXECAVALIACAO.NRAVALIACAO     = :P_NRAVALIACAO
                AND EXECAVALIACAO.NRCOMBQUESTAVAL = :P_NRCOMBQUESTAVAL
                AND EXECAVALIACAO.NRPARCEXECAVAL  = :P_NRPARCEXECAVAL
                AND EXECAVALIACAO.NRPARCAVALIADO  = :P_NRPARCAVALIADO)"
        );

        return $query->execute($params);;
    }
    
    public function getAvaliacoes($nrorg){
         $query = $this->entityManager->getConnection()->prepare("
            SELECT DISTINCT NMTIPOAVALIACAO,
                TO_CHAR(DTINIREALIZAVAL, 'DD/MM/YYYY') DTINIREALIZAVAL,
                EXECAVALIACAO.NRPARCEXECAVAL,
                EXECAVALIACAO.NRORG,
                EXECAVALIACAO.NRPARCAVALIADO,
                TO_CHAR(DTFIMREALIZAVAL, 'DD/MM/YYYY') DTFIMREALIZAVAL,
                PARTICIPAVAL.IDSITUACAOAVAL SITUACAO,
                EXECAVALIACAO.NRAVALIACAO,
                TO_CHAR(EXECAVALIACAO.DTEXECUCAOAVAL, 'DD/MM/YYYY') DTEXECUCAOAVAL,
                EXECAVALIACAO.NRCOMBQUESTAVAL,
                TIPORELACIONA.NMTIPORELACIONA,
                EXEC.NRPARCNEGOCIO NRAVALIADOR,
                EXEC.NMPRINCIPALPARC AVALIADOR,
                APLIC.NRPARCNEGOCIO NRAVALIADO,
                APLIC.NMPRINCIPALPARC AVALIADO,
                AVALIACAOGERAL.NMAVALIACAO,
                PARTICIPAVAL.VRRESULTAVALIA,
                TIPORESU.NMTIPORESU,
                CASE
                    WHEN SUB.NRPLANACAOAVL IS NOT NULL THEN 'S'
                    ELSE 'N'
                END NMPLANACAOAVL
                FROM EXECAVALIACAO,
                     AVALIACAOGERAL,
                     TIPOAVALIACAO,
                     TIPORELACIONA,
                     PARTICIPAVAL,
                     PARCNEGOCIO EXEC,
                                 PARCNEGOCIO APLIC,
                                 TIPORESU,
                                 PLANACAOAVL SUB
                        WHERE AVALIACAOGERAL.NRTIPOAVALIACAO = TIPOAVALIACAO.NRTIPOAVALIACAO
                          AND PARTICIPAVAL.NRTIPORELACIONA = TIPORELACIONA.NRTIPORELACIONA
                          AND PARTICIPAVAL.NRPARCAVALIADO = EXECAVALIACAO.NRPARCAVALIADO
                          AND PARTICIPAVAL.NRPARCEXECAVAL = EXECAVALIACAO.NRPARCEXECAVAL
                          AND PARTICIPAVAL.NRAVALIACAO = EXECAVALIACAO.NRAVALIACAO
                          AND EXEC.NRPARCNEGOCIO = EXECAVALIACAO.NRPARCEXECAVAL
                          AND APLIC.NRPARCNEGOCIO = EXECAVALIACAO.NRPARCAVALIADO
                          AND EXECAVALIACAO.NRORG = AVALIACAOGERAL.NRORG
                          AND EXECAVALIACAO.NRAVALIACAO = AVALIACAOGERAL.NRAVALIACAO
                          AND PARTICIPAVAL.CDTIPORESU = TIPORESU.CDTIPORESU(+)
                          AND SUB.NRAVALIACAO(+) = EXECAVALIACAO.NRAVALIACAO
                          AND SUB.NRPARCEXECAVAL(+) = EXECAVALIACAO.NRPARCEXECAVAL
                          AND SUB.NRPARCNEGOCIO(+) = EXECAVALIACAO.NRPARCAVALIADO
                          AND EXECAVALIACAO.NRORG = AVALIACAOGERAL.NRORG
                          AND PARTICIPAVAL.NRORG = EXECAVALIACAO.NRORG
                          AND EXEC.NRORG = EXECAVALIACAO.NRORG
                          AND APLIC.NRORG = EXECAVALIACAO.NRORG
                          AND (EXECAVALIACAO.NRORG = :P_NRORG ) ORDER BY AVALIADO" );
    
        $query->execute($nrorg);

        return $query->fetchAll();
    }
} 
