<?php
    
    namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico;
    
    use HCM\Util\MsgCalculo;

    use Doctrine\ORM\EntityManager;
    use HCM\Util\HCMParameters;
    use HCM\Util\FPA;
    use HCM\Util\DateUtil;
    use HCM\Util\Repositories;
    use HCM\Geral\Entity\GpevinculomEntity;
    use HCM\FPA\Entity\TipohierarqorgEntity;
    use HCM\FPA\Entity\HierarquiaorgEntity;
    use HCM\FPA\Entity\FpacalculofolhaEntity;
    use HCM\FPA\Entity\FpaHoleritemagnetEntity;
    use HCM\FPA\Entity\FpatipomovimentEntity;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\Holerite;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\FpaHoleritemagnet;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro\Registro3HoleriteSantander;
    
class PercorreHolerite{
    
 
    const PREFIXO_NOME_ARQUIVO = 'holerite';
    const EXTENSAO_ARQUIVO = '.txt';

    public function __construct($row, HCMParameters $hcmParameters,EntityManager $entityManager){

        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->nrorg = $this->hcmParameters->getNrorgTrabalho();
        $this->nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->nrEstrutTrab = FPA::getEstruturaTrabalho();
        $this->mesCompetencia = DateUtil::getDataDeString(FPA::getDtCompetencia(),DateUtil::FORMATO_BRASILEIRO,true);
        $this->nrTipoModalidadeCalculo = FPA::getNrModalidadeCalculo();
        $this->nrOcorrenciaCalculo = FPA::getOcorrenciaCalculo();
        
        $this->mensagens = array();
        
        $this->vinculom = new GpevinculomEntity($this->entityManager,$this->hcmParameters);
        $this->calculoFolha = new FpacalculofolhaEntity($this->entityManager,$this->hcmParameters);
        $this->holeriteMagnetico = new FpaHoleritemagnet($row,$this->hcmParameters,$this->entityManager);

        $row = $row[0];
        $this->inicializaVariaveis($row);

        $tpProcessamento = strtoupper($row['TPPROCESSAMENTO']);
        switch ($tpProcessamento){
        case 'COMPETENCIA_MENSAL' :
             $this->retrocesso = false;
             break;
        case 'RETROCESSO' :
            $this->retrocesso = true;
            break;
        default:
            throw new \Exception("Tipo de Processo Invalido");
        }            

        $this->formulario = $this->entityManager->getRepository(Repositories::FPA_FORMHOLERITE)->findOneBy(array("nrformholerite"=>$row['NRFORMHOLERITE'],"nrorg"=>$row['NRORG']));

        $this->holerite = new Holerite(
            $this->nrorg, $this->nrorgPadrao, $this->mesCompetencia, $this->holeriteMagnetico, $this->formulario, $this->nrCalculoFolha,$this->entityManager,$this->hcmParameters
        );
    }

    public function inicializaVariaveis($row) {
            $this->dsInicio                 = $row['DSINICIO'];
            $this->dsFim                    = $row['DSFIM'];
            $this->dsOrdem                  = $row['DSORDEM'];
            $this->nrTpHierarquiaOrg        = $row['NRHIERARQORG'];
            $this->idProcessaVinculo        = $row['IDPROCVINCULO'];
            $this->idGeraMeioMagnetico      = $row['IDGERAMEIOMAG'];
            $this->filePath                 = $row['DSCAMINHOARQUIVO'];
            $this->nrEstruturaResp          = $row['NRESTRUTRESPONS'];
            $this->tpInscResp               = $row['IDTPINSCRESPONS'];
            $this->nrEvento                 = $row['NREVENTOM'];
            $this->nrTipoMovimento          = $row['NRTIPOMOVIMENT'];
            $this->cdBanco                  = $row['CDBANCO'];
            $this->nrGrupoHolerite          = $row['NRGRUPOHOLERITE'];

        $calculoFolha = $this->calculoFolha->retornaCalculoFolha(
            $this->nrorg,
            $this->nrTipoMovimento,
            $this->mesCompetencia,
            $this->nrTipoModalidadeCalculo,
            $this->nrOcorrenciaCalculo
        );
        
        $this->nrCalculoFolha = end($calculoFolha)->getNrcalculofolha();

        $this->msgSucessoPersisteMagnetico = null;
        $this->msgSucessoGeraMagnetico = null;
    }

    public function percorre(){
        set_time_limit(0);
        $return = array();

        $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculo($this->nrCalculoFolha, $this->hcmParameters->getNrorgTrabalho(), 'HORELITE');
        
        if (!$this->retrocesso) {

            $resultado = $this->geraHolerite();
            if ($resultado === null) {
                throw new \Exception('Erro ao gerar o arquivo');
            }
            else if ($resultado['ABORT'] === true) {
                $mensagens = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $this->nrCalculoFolha,
                'nrorg' => $this->hcmParameters->getNrorgTrabalho(),'dsorigmsg' => 'HORELITE'));
                $return['MENSAGENS'] = $mensagens;
                $return['DSSTATUS'] = 'Documento eletrônico com erros. Não finalizado';
            }
            else if ($resultado['ERRO'] === true) {
                $mensagens = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $this->nrCalculoFolha,
                'nrorg' => $this->hcmParameters->getNrorgTrabalho(),'dsorigmsg' => 'HORELITE'));
                $return['MENSAGENS'] = $mensagens;
                if($resultado['ARQUIVO_ELETRONICO']){
                    $return['ARQUIVO_ELETRONICO'] = $resultado['ARQUIVO_ELETRONICO'];
                    $return['DSSTATUS'] = 'Documento eletrônico finalizado com advertências.<br>Verifique as mensagens.';
                }else {
                    $return['DSSTATUS'] = $resultado['MSGMAGNETICO'];
                }
            } else{
                $return['DSSTATUS'] = 'Documento eletrônico finalizado com sucesso';
                if($resultado['ARQUIVO_ELETRONICO'])
                    $return['ARQUIVO_ELETRONICO'] = $resultado['ARQUIVO_ELETRONICO'];
            }
            
        } else {
            $this->holerite->retroceder();
            $return['DSSTATUS'] = 'Retrocedido com sucesso';
        }
        return $return;
    }

    private function geraHolerite(){
        try{
            $msgProcessamento = false;
            $msgMagnetico = false;
            $idabort = false;
            $idstatus = MsgCalculo::ID_PENDENTE;
            
            if ($this->idProcessaVinculo === "S") {
                $this->holeriteMagnetico->inicializaListaRegistros(
                    $this->mesCompetencia, RegistroHolerite::LISTA_TIPO_REGISTROS_UNICO, $this->nrorg, $this->cdBanco, $this->nrTipoMovimento
                );
                $vinculos = $this->vinculom->getVinculosAtivosParaEvento(
                    $this->nrorg, $this->mesCompetencia, $this->dsInicio, $this->dsFim, $this->nrEvento,
                    $this->nrTipoMovimento, $this->dsOrdem
                );
                
                // Percorre os vinculos gerando o Holerite Eletronico.
                $qdeVinc = count($vinculos);
                $cont = 0;
                foreach ($vinculos as $vinculo) {
                    $cont++;

                    MsgCalculo::getInstance()->setDados($this->nrCalculoFolha, $vinculo->getNrvinculom(), $vinculo->getNrorg(), 'HORELITE');
                    
                    $this->holerite->preparaHolerite($vinculo, $qdeVinc == $cont);
                    
                    $msgcalculo = MsgCalculo::getInstance()->getDados();
                    
                    if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
                        $idstatus = MsgCalculo::ID_COM_ERRO;
                    } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $idstatus != MsgCalculo::ID_COM_ERRO) {
                        $idstatus = MsgCalculo::ID_COM_ALERTA;
                    }
                    
                    if ($msgcalculo['IDABORT'] === true) {
                        $idabort = true;
                        break;
                    } else {
                        MsgCalculo::getInstance()->salvaMensagens(true);
                    }
                    MsgCalculo::getInstance()->clear();
                }
                
                $qtdRegistros = count($this->holeriteMagnetico->getItems());
                if ($qtdRegistros > 0) { 
                    $persisteHolerite = $this->holeriteMagnetico->persisteItems();
                }
            }
        if (($idabort == false) and ($idstatus !== MsgCalculo::ID_COM_ERRO)) {
            if ($this->idGeraMeioMagnetico === "S") {
                try {
                    $arquivo = $this->geraMagnetico();
                } catch (\Exception $e) {
                    return array(
                        "MSGMAGNETICO" => "Erro ao gerar magnético." . $e->getMessage(),
                        "ERRO" => true,
                        "ABORT" => $idabort,
                        "ARQUIVO_ELETRONICO" => false
                    );
                }
            }
        }
        
        return array(
            "ERRO" => (($idstatus === MsgCalculo::ID_COM_ERRO) or ($idstatus === MsgCalculo::ID_COM_ALERTA)),
            "ABORT" => $idabort,
            "ARQUIVO_ELETRONICO" => isset($arquivo)?$arquivo:false
        );
        }
        catch(Exception $e){
            throw $e;
        }
    }


    private function geraMagnetico(){
        $resultado = $this->holeriteMagnetico->retornaRegistros($this->mesCompetencia, $this->nrorg, $this->cdBanco, $this->nrTipoMovimento);
        if (count($resultado) > 0) {
            $this->constroiNomeArquivo();
        

            $nomePasta = "ARQUIVO_ELETRONICO";
            $nomeArquivo = $this->nomeArquivo;
            $extensao = self::EXTENSAO_ARQUIVO;
            $arquivo = array();
            
            $this->contador = 1;
            
            //for($c = 0; $c<count($resultado);$c++){
              //  $linhaRegistro = $this->modificaLinhas($resultado[$c]);
             //   $this->entityManager->persist($resultado[$c]);
            //    $arquivo[] = str_pad($c+1, 6, '0', STR_PAD_LEFT).$linhaRegistro."\n";
            //}
            
            foreach ($resultado as $item) {
                $linhaRegistro = $this->modificaLinhas($item);
                $this->entityManager->persist($item);
                $arquivo[] = $linhaRegistro."\r";
            }

            FPA::escreveArquivo($nomeArquivo,$arquivo,true);

            return FPA::getTempDirectory().'/'.$nomeArquivo;
        }else{
            throw new \Exception('Não existem registros processados para estes parametros');
        }
    }


    private function modificaLinhas($itemHoleriteMagnetico) {
        $linhaRegistro = $itemHoleriteMagnetico->getDslinharegistro();
        $tipoRegistro = $itemHoleriteMagnetico->getIdtiporegistro();

        if ($tipoRegistro === Registro3HoleriteSantander::REGISTRO_3) {
            $itemHoleriteMagnetico->setDslinharegistro($linhaRegistro);
            $this->contador++;
        } else {
            $linhaRegistro = $this->holerite->modificaLinhasConsolidacao($tipoRegistro, $linhaRegistro);
            $itemHoleriteMagnetico->setDslinharegistro($linhaRegistro);
        }

        return $linhaRegistro;
    }


    private function constroiNomeArquivo() {
        $ultimoSequencial = $this->holeriteMagnetico->retornaUltimoSequencialRegistro0($this->mesCompetencia,
            $this->nrorg, $this->cdBanco);

        if (!empty($ultimoSequencial)) {
            $sequencia = $ultimoSequencial;
            $sequencia++;
        } else {
            $sequencia = '11';
        }
        $this->nomeArquivo = 'ARQUIVO_ELETRONICO'.(string)$sequencia.$this->mesCompetencia->format("m").self::EXTENSAO_ARQUIVO;

    }


}