<?php
namespace HCM\FPA\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\NovoCodigo;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class ContabilizacaoMensalRepository {

    protected  $connection;
    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    /**
     * @return array
     */
 
    public function retrocedeResumoCTBL($nr_estrutura_sup, $tpmoviment) {
        $nrorg = $this->hcmParameters->getNrOrgTrabalho();
        $competencia = $this->hcmParameters->getDtCompetencia();
        
        $query = $this->entityManager->getConnection()->prepare("
          DELETE FROM FPA_RESUMOCTBL WHERE NRORG = :P_NRORG AND NRESTRUTURAMSUP = :P_NRESTRUTURAMSUP AND DTOCORRENCIA = :P_COMPETENCIA AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT
        ");

        $return = $query->execute(
            array(
                "P_NRORG"            => $nrorg,
                "P_NRESTRUTURAMSUP" => $nr_estrutura_sup,
                "P_COMPETENCIA" => $competencia,
                "P_NRTIPOMOVIMENT" => $tpmoviment
            )
        );
        return $return;
    }
    
    public function retrocedeResumoDRO($nr_estrutura_sup, $tpmoviment) {
        $nrorg = $this->hcmParameters->getNrOrgTrabalho();
        $competencia = $this->hcmParameters->getDtCompetencia();
        
        $query = $this->entityManager->getConnection()->prepare("
          DELETE FROM FPA_RESUMODRO WHERE NRORG = :P_NRORG AND NRESTRUTURAMSUP = :P_NRESTRUTURAMSUP AND DTOCORRENCIA = :P_COMPETENCIA AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT
        ");

        $return = $query->execute(
            array(
                "P_NRORG"            => $nrorg,
                "P_NRESTRUTURAMSUP" => $nr_estrutura_sup,
                "P_COMPETENCIA" => $competencia,
                "P_NRTIPOMOVIMENT" => $tpmoviment
            )
        );
        return $return;
    }
    
    public function getContabilizacao($nrorg, $dtmescompetenc, $dsordem, $dsinicio, $dsfim, $nrtipomoviment, $nrtpmodalidcal, $nrocorrecal) {
        
        $condIntervalo = "AND 1=2";
        if ($dsordem == 'LISTA_VINCULO') {
            $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
            $condIntervalo = "AND VM.NRVINCULOM IN ($dsinicio)";
        }
        if ($dsordem == 'LISTA_ESTRUTURA') {
            $dsinicio = is_array($dsinicio) ? implode(',',$dsinicio) : $dsinicio;
            $condIntervalo = "AND VH.NRESTRUTLEGAL IN ($dsinicio)";
            if($dsfim){
                $dsfim = is_array($dsfim) ? implode(',',$dsfim): $dsfim;
                $condIntervalo .= "AND VH.NRESTRUTGEREN IN ($dsfim)";
            }
        }
        if ($dsordem == 'LISTA_NRESTRUTSIND') {
            $condIntervalo = "AND VH.NRESTRUTLEGAL = $dsinicio";
            if($dsfim){
                $dsfim = is_array($dsfim) ? implode(',',$dsfim) : $dsfim;
                $condIntervalo .= " AND VH.NRESTRUTSIND IN ($dsfim)";
            }
        }
        
        if ($nrocorrecal == '') {
            $listaOcorrencias = '';
        } else {
            $listaOcorrencias = "AND CF.NROCORRECAL IN ($nrocorrecal)";
        }
        
        $query = $this->connection->prepare("
SELECT VH.NRESTRUTLEGAL AS NRESTRUTURAMSUP,
       DECODE(VM.NRORG,20,VH.NRESTRUTLEGAL,VH.NRESTRUTGEREN) AS NRESTRUTURAM,
       EC.CDCONTCTBLDEB, EC.CDHISTPADRDEB, EC.CDCONTCTBLCRE, EC.CDHISTPADRCRE,
       SUM(IT.VREVENTOFOLHA) AS VRLANCAMENTO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRORG = VH.NRORG
                                                  AND H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN ESTRUTURAM EG ON EG.NRORG = VH.NRORG
                    AND EG.NRESTRUTURAM = VH.NRESTRUTGEREN
  JOIN FPA_ITECALCFOLHA IT ON IT.NRORG = VM.NRORG
                          AND IT.NRVINCULOM = VM.NRVINCULOM
  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IT.NRORG
                          AND CF.NRCALCULOFOLHA = IT.NRCALCULOFOLHA
                          AND CF.NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT
                          AND CF.NRTPMODALIDCAL = :P_NRTPMODALIDCAL
                          AND CF.DTOCORRENCIA = :P_DTMESCOMPETENC
                          $listaOcorrencias
  JOIN (SELECT DISTINCT EC.NRORG, EC.NREVENTOM, EC.NRTIPOMOVIMENT, EC.NRESTRUTURAM, EC.NRTPEVENTOCTBL,
               EC.CDCONTCTBLDEB, EC.CDHISTPADRDEB, EC.CDCONTCTBLCRE, EC.CDHISTPADRCRE
          FROM FPA_EVENTOCTBL EC
         WHERE EC.NRORG = :P_NRORG) EC ON EC.NRORG = IT.NRORG
                                      AND EC.NREVENTOM = IT.NREVENTOM
                                      AND (EC.NRTIPOMOVIMENT = CF.NRTIPOMOVIMENT OR EC.NRTIPOMOVIMENT IS NULL)
                                      AND (EC.NRESTRUTURAM = VH.NRESTRUTLEGAL OR EC.NRESTRUTURAM IS NULL)
                                      AND (EC.NRTPEVENTOCTBL = EG.NRTPEVENTOCTBL OR EC.NRTPEVENTOCTBL IS NULL)
 WHERE VM.NRORG = :P_NRORG
        /*VINCULOS ATIVOS*/
   AND (((VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
         AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)) OR
        /*VINCULOS RESCISAO COMPLEMENTAR*/
        (VM.DTRESCISAOVINC IS NOT NULL AND VM.DTRESCISAOVINC < :P_DTMESCOMPETENC
         AND EXISTS (SELECT 1 FROM FPA_ITECALCFOLHA IC
                       JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IC.NRORG
                                               AND CF.NRCALCULOFOLHA = IC.NRCALCULOFOLHA
                      WHERE CF.DTOCORRENCIA = :P_DTMESCOMPETENC
                        AND IC.NRORG = :P_NRORG
                        AND CF.NRTPMODALIDCAL = 1
                        AND IC.NREVENTOM = 3050
                        AND IC.NRVINCULOM = VM.NRVINCULOM)))
   /*CRITERIOS DE SELECAO*/
   AND ((:P_DSORDEM = 'ALFABETICA' AND TRIM(UPPER(VM.NMVINCULOM)) BETWEEN :P_DSINICIO AND :P_DSFIM) OR
        (:P_DSORDEM = 'MATRICULA' AND VM.CDMATRICULA BETWEEN :P_DSINICIO AND :P_DSFIM) OR
        (:P_DSORDEM = 'DATA_ADMISSAO' AND VM.DTADMISSAOVINC BETWEEN :P_DSINICIO AND :P_DSFIM) OR
        (:P_DSORDEM = 'ESTRUTURA' AND VH.NRESTRUTLEGAL = :P_DSINICIO AND (VH.NRESTRUTGEREN = :P_DSFIM OR :P_DSFIM IS NULL)) OR
        (:P_DSORDEM = 'LISTA_VINCULO' $condIntervalo) OR
        (:P_DSORDEM = 'LISTA_ESTRUTURA' $condIntervalo) OR
        (:P_DSORDEM = 'LISTA_NRESTRUTSIND' $condIntervalo))
 GROUP BY VH.NRESTRUTLEGAL, DECODE(VM.NRORG,20,VH.NRESTRUTLEGAL,VH.NRESTRUTGEREN),
       EC.CDCONTCTBLDEB, EC.CDHISTPADRDEB, EC.CDCONTCTBLCRE, EC.CDHISTPADRCRE
 ORDER BY VH.NRESTRUTLEGAL, DECODE(VM.NRORG,20,VH.NRESTRUTLEGAL,VH.NRESTRUTGEREN)"
        );

        $query->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_DTMESCOMPETENC' => $dtmescompetenc,
                'P_DSORDEM' => $dsordem,
                'P_DSINICIO' => $dsinicio,
                'P_DSFIM' => $dsfim,
                'P_NRTIPOMOVIMENT' => $nrtipomoviment,
                'P_NRTPMODALIDCAL' => $nrtpmodalidcal,
            )
        );
        
        return $query->fetchAll();
    }
    

    
}