<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use Geral\Util\NovoCodigo;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\FPA\Repository\AprovacaoCalculoRepository;
use HCM\Util\Repositories;
use Teknisa\Model\Gestaopessoas\Entities\FpaItecalcfolha;

class AprovacaoCalculoService {
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, AprovacaoCalculoRepository $aprovacaoCalculoRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->aprovacaoCalculoRepository = $aprovacaoCalculoRepository;
    }
    
    public function getVinculoEventosNaoAprovados($parameters, $conditions){
        return $this->aprovacaoCalculoRepository->getVinculoEventosNaoAprovados($parameters, $conditions);
    }
    
    public function aprovaCalculo($row){
        try{
            $this->entityManager->beginTransaction();
            
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $codigo = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'FPA_ITECALCFOLHA', 1, 12, true);
            
            $obj = Repositories::FPA_ITECALCFOLHA;
            $obj = new $obj();
            
            $obj->setNritemcalcfolha($codigo);
            $obj->setNrcalculofolha($row['NRCALCULOFOLHA']);
            $obj->setNrvinculom($row['NRVINCULOM']);
            $obj->setNreventom('3035');
            $obj->setIdevendemoncalc('S');
            $obj->setVrreffolha(0);
            $obj->setVreventofolha(0.0);
            $obj->setDtocorrencia(DateUtil::getDataDeString(substr($row['DTOCORRENCIA'], 0, 10)));
            $obj->setQtocorproxperi(1);
            $obj->setIdrefevalorcalc('NORMAL');
            $obj->setVrvelho(null);
            $obj->setNrlotelancevento(null);
            $obj->setNrorg($nrorg);
            $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            
            $this->entityManager->persist($obj);
            $this->entityManager->flush($obj);
            
            $this->entityManager->commit();
            return array('NRVINCULOM'      => $row['NRVINCULOM'],
                         'NRCALCULOFOLHA'  => $row['NRCALCULOFOLHA'],
                         "NRITEMCALCFOLHA" => $codigo);
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
        
}
