<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\Util\HCMParameters;
use HCM\Geral\Services\EsoEventoService;

class EsoEventoController extends Crud {

    protected $dataSourceNameGet = 'getEsoEvento';
    protected $dataSourceNameRegistoGet = 'getEsoRegistro';
    protected $dataSourceName = 'getEventosESocial';
    
    protected $parameterBagColumns = array('P_IDATIVO', 'P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EsoEventoService $esoEventoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->esoEventoService = $esoEventoService;
        parent::__construct($dataSourceManager);
    }

   public function getEsoEvento (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            foreach($rows as $key => &$row){
                
                $row['NREVPRECED'] = explode(",", $row['NREVPRECED']);
                foreach($row['NREVPRECED'] as $chave => $value)
                    $row['NREVPRECED'][$chave] = intval($value);
                
                $row['NMEVPRECED'] = explode(",", $row['NMEVPRECED']);
                
                $row['NMEVDEPEND'] = explode(",", $row['NMEVDEPEND']);
                
                $row['NREVDEPEND'] = explode(",", $row['NREVDEPEND']);
                foreach($row['NREVDEPEND'] as $chave => $value)
                    $row['NREVDEPEND'][$chave] = intval($value);
            
            }
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function setEsoEvento(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NREVENTO', $row)) {
                    if ($row['NREVENTO'] === '') {
                        $row['NREVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_EVENTO', 1, 12, true); 
                    }
                } else {
                    $row['NREVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_EVENTO', 1, 12, true); 
                }
            }
            
            if(is_array($row['NMEVPRECED'])){
                $row['NMEVPRECED'] = implode(',',$row['NMEVPRECED']);
            }
                    
            if(is_array($row['NMEVDEPEND'])){
                $row['NMEVDEPEND'] = implode(',',$row['NMEVDEPEND']);
            }
            
            if(!$row['NREVPRECED']){
                $row['NMEVPRECED'] = null;
            }
            
            if(!$row['NREVDEPEND']){
                $row['NMEVDEPEND'] = null;
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteEsoEvento(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function getEsoRegistro (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameRegistoGet);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $dataSet->getRows()));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function setEsoRegistro(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCAMPO', $row)) {
                    if ($row['NRCAMPO'] === '') {
                        $row['NRCAMPO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_REGISTRO', 1, 12, true); 
                    }
                } else {
                    $row['NRCAMPO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_REGISTRO', 1, 12, true); 
                }
            }
            $request->setDataSourceName($this->dataSourceNameRegistoGet);
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteEsoRegistro(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $request->setDataSourceName($this->dataSourceNameRegistoGet);
             parent::delete($request, $response);
         } catch(\Exception $e){
             $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
         }    
     }
    
    public function setEsoImportacaoXSD(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->esoEventoService->importacaoXSD($row['NRORG'], $row['NREVENTO'], $row['DSPATH']);
            }
            $response->addMessage(new DTO\Response\Message('Importação realizada com sucesso', DTO\Response\Message::TYPE_MESSAGE));
        } catch(\Exception $e) {
            if ($e->getCode() == 1) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 