<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 21/09/2016
 * Time: 11:14
 */

namespace HCM\SST\Controllers;

use HCM\Util\Messages;
use HCM\SST\Services\ProgExamePeriodService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ProgExamePeriodController extends Crud {

    protected $dataSourceName = 'sst_formprogexames';
    protected $dataSourceName_get = 'sst_formprogexames_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERINCLUSAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ProgExamePeriodService $progExamePeriodService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->progExamePeriodService = $progExamePeriodService;
        parent::__construct($dataSourceManager);
    }

    public function getFormprogexames(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setFormprogexames(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (!array_key_exists('DTLIMITE', $row)) {
                    $row['DTLIMITE'] = null;
                }
                if (array_key_exists('NRFORMPROGEXAMES', $row)) {
                    if ($row['NRFORMPROGEXAMES'] === '') {
                        $row['NRFORMPROGEXAMES'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_FORMPROGEXAMES', 1, 12, true);
                    }
                } else {
                    $row['NRFORMPROGEXAMES'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_FORMPROGEXAMES', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFormprogexames(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getSimulacaoProgExames(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $dsordem = null;
            $dsinicio = null;
            $dsfim = null;
            $qtminatendmedio = null;
            $qtatendavaliacao = null;
            $qthrdiaavaliacao = null;
            $dtinicial = null;
            $dtfinal = null;
            foreach ($request->getFilterCriteria()->getConditions() as $condition) {
                $dsordem = $condition['columnName'] == 'DSORDEM' ? $condition['value'] : $dsordem;
                $dsinicio = $condition['columnName'] == 'DSINICIO' ? $condition['value'] : $dsinicio;
                $dsfim = $condition['columnName'] == 'DSFIM' ? $condition['value'] : $dsfim;
                $qtminatendmedio = $condition['columnName'] == 'QTMINATENDMEDIO' ? $condition['value'] : $qtminatendmedio;
                $qtatendavaliacao = $condition['columnName'] == 'QTATENDAVALIACAO' ? $condition['value'] : $qtatendavaliacao;
                $qthrdiaavaliacao = $condition['columnName'] == 'QTHRDIAAVALIACAO' ? $condition['value'] : $qthrdiaavaliacao;
                $dtinicial = $condition['columnName'] == 'DTINICIAL' ? substr($condition['value'], 0, 10) : $dtinicial;
                $dtfinal = $condition['columnName'] == 'DTFINAL' ? substr($condition['value'], 0, 10) : $dtfinal;
            }

            $dataSet = $this->progExamePeriodService->getSimulacao($dsordem, $dsinicio, $dsfim, $qtminatendmedio, $qtatendavaliacao, $qthrdiaavaliacao, $dtinicial, $dtfinal);
            if(empty($dataSet)) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM05000_SST_PROGEXAMES_DADOS_NOT_FOUND_ERROR, DTO\Response\Message::TYPE_ERROR));
            }
            $response->addDataSet(new DataSet('getSimulacaoProgExames', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function gravarProgramacaoExames(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dsordem = $row['DSORDEM'];
                $dsinicio = $row['DSINICIO'];
                $dsfim = $row['DSFIM'];
                $qtminatendmedio = $row['QTMINATENDMEDIO'];
                $qtatendavaliacao = $row['QTATENDAVALIACAO'];
                $qthrdiaavaliacao = $row['QTHRDIAAVALIACAO'];
                $dtinicial = substr($row['DTINICIAL'], 0, 10);
                $dtfinal = substr($row['DTFINAL'], 0, 10);

                $this->progExamePeriodService->gravar($dsordem, $dsinicio, $dsfim, $qtminatendmedio, $qtatendavaliacao, $qthrdiaavaliacao, $dtinicial, $dtfinal);
                $response->addNotification(new DTO\Response\Notification('gravarProgramacaoExames'), DTO\Response\Notification::TYPE_SUCCESS);
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}