<?php

namespace HCM\TRE\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class CursoController extends Crud {

    protected $dataSourceName = '';
    protected $dataSourceName_perfil = 'tre_curso';
    protected $dataSourceName_perfil_get = 'getCurso_Tre';
    protected $dataSourceName_conteudo = 'tre_conteudocurso';
    protected $dataSourceName_conteudo_get = 'getConteudoCurso_Tre';
    protected $dataSourceName_sup = 'tre_conteudosup';
    protected $dataSourceName_curso = 'tre_curso';
    protected $dataSourceName_tipo = 'tipocurso';
    protected $dataSourceName_config = 'configcurso';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters , \HCM\TRE\Services\CursoService $cursoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->CursoService = $cursoService;
        parent::__construct($dataSourceManager);
    }
    
    public function getTipoCurso_Tre(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_tipo);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getConfigCurso_Tre(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_config);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function getCurso_Tre(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_perfil_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as &$row){
            if(strlen($row['QTCARGAHORARIA']) == 1) $row['QTCARGAHORARIA'] = '0000'.$row['QTCARGAHORARIA'];
            elseif(strlen($row['QTCARGAHORARIA']) == 2) $row['QTCARGAHORARIA'] = '000'.$row['QTCARGAHORARIA'];
            elseif(strlen($row['QTCARGAHORARIA']) == 3) $row['QTCARGAHORARIA'] = '00'.$row['QTCARGAHORARIA'];
            elseif(strlen($row['QTCARGAHORARIA']) == 4) $row['QTCARGAHORARIA'] = '0'.$row['QTCARGAHORARIA'];
        }
        $response->addDataSet(new DataSet($this->dataSourceName_perfil_get, $rows));
    }

    
    public function setCurso_Tre(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceName_perfil;
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['CDESCOLARIDADE'] = isset($row['CDESCOLARIDADE'])? $row['CDESCOLARIDADE'] : null;
                if (array_key_exists('NRCURSO', $row)) {
                    if ($row['NRCURSO'] === '') {
                        $row['NRCURSO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'GPE_CURSO', 1, 12, true);
                    }
                } else {
                    $row['NRCURSO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'GPE_CURSO', 1, 12, true);
                }
                $row['QTCARGAHORARIA'] = str_replace(":", "", $row['QTCARGAHORARIA']);
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteCurso_Tre(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceName_perfil;
            $row = $request->getDataSet()->getRows()[0];
            $res = $this->CursoService->ConteudoExists($row['NRORG'],$row['NRCURSO']);
            if($res){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Existem conteúdos para este curso, favor deletá-los","E"));
            }else{
                parent::delete($request, $response);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function getConteudoCurso_Tre(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        if($filterData->getConditions()[0]['columnName'] == 'id')
            return;
        $filterData->setDataSourceName($this->dataSourceName_conteudo_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as &$row){
            if(strlen($row['QTHORCONTEUDOCURSO']) == 1) $row['QTHORCONTEUDOCURSO'] = '0000'.$row['QTHORCONTEUDOCURSO'];
            elseif(strlen($row['QTHORCONTEUDOCURSO']) == 2) $row['QTHORCONTEUDOCURSO'] = '000'.$row['QTHORCONTEUDOCURSO'];
            elseif(strlen($row['QTHORCONTEUDOCURSO']) == 3) $row['QTHORCONTEUDOCURSO'] = '00'.$row['QTHORCONTEUDOCURSO'];
            elseif(strlen($row['QTHORCONTEUDOCURSO']) == 4) $row['QTHORCONTEUDOCURSO'] = '0'.$row['QTHORCONTEUDOCURSO'];
        }
        $response->addDataSet(new DataSet($this->dataSourceName_conteudo_get, $rows));
    }
    
    public function getConteudoSup_Tre(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_sup);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setConteudoCurso_Tre(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
        $this->dataSourceName = $this->dataSourceName_conteudo;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCONTEUDOCURSO', $row)) {
                    if ($row['NRCONTEUDOCURSO'] === '') {
                        $row['NRCONTEUDOCURSO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'GPE_CONTEUDOCURSO', 1, 12, true);
                    }
                } else {
                    $row['NRCONTEUDOCURSO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORG'], 'GPE_CONTEUDOCURSO', 1, 12, true);
                }
                $row['QTHORCONTEUDOCURSO'] = str_replace(":", "", $row['QTHORCONTEUDOCURSO']);
            }
            $row['NRCONTEUDOCURSOSUP'] = isset($row['NRCONTEUDOCURSOSUP'])? $row['NRCONTEUDOCURSOSUP'] : null;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteConteudoCurso_Tre(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceName_conteudo;
            $row = $request->getDataSet()->getRows()[0];
            $res = $this->CursoService->ConteudoInfExists($row['NRORG'],$row['NRCURSO'],$row['NRCONTEUDOCURSO']);
            if($res){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Existem conteúdos inferiores a este, favor deletá-los","E"));
            }else{
                parent::delete($request, $response);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 